# initial_ontology_definitions.py

"""
Decides, if classes and properties are defined before adding new data to the knowledge graph. Can be 
set with a python file which contains a number class and property definitions, executed by rdflib. These 
definitions are then executed in the mapping scripts before any data is being mapped from OMA tables.

@author: Philipp Schneider
"""

from rdflib import Graph, URIRef
from rdflib.namespace import RDF, RDFS, Namespace, OWL, DCTERMS

def create_initial_classes_and_properties_heraldry(g, dho_n):
    """
    Helper function for the module `map-tblBranch.py`. Creates classes and properties related to heraldic descriptions, before
    any data is added to the Knowledge Graph by `map-tblBranch.py`.
    """
    # Define initial classes
    g.add((dho_n.CoatOfArms, RDF.type, OWL.Class))
    g.add((dho_n.Field, RDF.type, OWL.Class))
    g.add((dho_n.Pattern, RDF.type, OWL.Class))
    g.add((dho_n.Layer, RDF.type, OWL.Class))
    g.add((dho_n.Charge, RDF.type, OWL.Class))
    g.add((dho_n.Seme, RDF.type, OWL.Class))
    g.add((dho_n.Seme, RDFS.subClassOf, dho_n.Charge))
    g.add((dho_n.Direction, RDF.type, OWL.Class))
    g.add((dho_n.Line, RDF.type, OWL.Class))
    g.add((dho_n.Arrangement, RDF.type, OWL.Class))
    g.add((dho_n.Modifier, RDF.type, OWL.Class))
    g.add((dho_n.LineModifier, RDFS.subClassOf, dho_n.Modifier))

    # Tincture classes
    g.add((dho_n.Tincture, RDF.type, OWL.Class))
    g.add((dho_n.Metal, RDFS.subClassOf, dho_n.Tincture))
    g.add((dho_n.Colour, RDFS.subClassOf, dho_n.Tincture))
    g.add((dho_n.Stain, RDFS.subClassOf, dho_n.Tincture))
    g.add((dho_n.Fur, RDFS.subClassOf, dho_n.Tincture))

    g.add((dho_n.Argent, RDFS.subClassOf, dho_n.Metal))
    g.add((dho_n.Or, RDFS.subClassOf, dho_n.Metal))
    g.add((dho_n.Azure, RDFS.subClassOf, dho_n.Colour))
    g.add((dho_n.Gules, RDFS.subClassOf, dho_n.Colour))
    g.add((dho_n.Purpure, RDFS.subClassOf, dho_n.Colour))
    g.add((dho_n.Sable, RDFS.subClassOf, dho_n.Colour))
    g.add((dho_n.Vert, RDFS.subClassOf, dho_n.Colour))
    g.add((dho_n.Vair, RDFS.subClassOf, dho_n.Fur))
    g.add((dho_n.Vairy, RDFS.subClassOf, dho_n.Fur))
    g.add((dho_n.Ermine, RDFS.subClassOf, dho_n.Fur))
    g.add((dho_n.Tenne, RDFS.subClassOf, dho_n.Stain))

    g.add((dho_n.InOmbre, RDFS.subClassOf, dho_n.Tincture))
    g.add((dho_n.Naturel, RDFS.subClassOf, dho_n.Tincture))
    g.add((dho_n.Missing, RDFS.subClassOf, dho_n.Tincture))

    # Special classes
    g.add((dho_n.Multiple, RDF.type, OWL.Class))

    g.add((dho_n.MarshallingType, RDF.type, OWL.Class))

    g.add((dho_n.BlazonTermDefinition, RDF.type, OWL.Class))

    # Define initial properties
    g.add((dho_n.hasOMAid, RDF.type, OWL.ObjectProperty))
    g.add((dho_n.hasCoatOfArms, RDF.type, OWL.ObjectProperty))
    g.add((dho_n.hasField, RDF.type, OWL.ObjectProperty))
    g.add((dho_n.hasPattern, RDF.type, OWL.ObjectProperty))

    g.add((dho_n.hasLayer, RDF.type, OWL.ObjectProperty))
    g.add((dho_n.hasLayer1, RDFS.subPropertyOf, dho_n.hasLayer))
    g.add((dho_n.hasLayer2, RDFS.subPropertyOf, dho_n.hasLayer))
    g.add((dho_n.hasLayer3, RDFS.subPropertyOf, dho_n.hasLayer))
    g.add((dho_n.hasLayer4, RDFS.subPropertyOf, dho_n.hasLayer))
    g.add((dho_n.hasLayerOverall, RDFS.subPropertyOf, dho_n.hasLayer))
    g.add((dho_n.hasCharge, RDF.type, OWL.ObjectProperty))
    g.add((dho_n.hasSeme, RDFS.subPropertyOf, dho_n.hasCharge))

    g.add((dho_n.hasVisualRelation, RDF.type, OWL.ObjectProperty))

    g.add((dho_n.hasDirection, RDF.type, OWL.ObjectProperty))
    g.add((dho_n.hasLine, RDF.type, OWL.ObjectProperty))
    g.add((dho_n.hasNumber, RDF.type, OWL.ObjectProperty))
    g.add((dho_n.hasPatternNumber, RDFS.subPropertyOf, dho_n.hasNumber))
    g.add((dho_n.arranged, RDF.type, OWL.ObjectProperty))
    g.add((dho_n.hasModifier, RDF.type, OWL.ObjectProperty))
    g.add((dho_n.hasLineModifier, RDFS.subPropertyOf, dho_n.hasModifier))
    g.add((dho_n.hasText, RDF.type, OWL.ObjectProperty))
    g.add((dho_n.hasLetter, RDF.type, OWL.ObjectProperty))

    g.add((dho_n.hasTincture, RDF.type, OWL.ObjectProperty))
    g.add((dho_n.hasTincture1, RDFS.subPropertyOf, dho_n.hasTincture))
    g.add((dho_n.hasTincture2, RDFS.subPropertyOf, dho_n.hasTincture))
    g.add((dho_n.hasTincture3, RDFS.subPropertyOf, dho_n.hasTincture))
    g.add((dho_n.hasTincture4, RDFS.subPropertyOf, dho_n.hasTincture))

    g.add((dho_n.numberOfMarshalledParts, RDFS.subPropertyOf, dho_n.hasNumber))
    g.add((dho_n.marshalledByType, RDF.type, OWL.ObjectProperty))
    g.add((dho_n.hasMarshalledPart, RDFS.subPropertyOf, dho_n.hasCoatOfArms))
    g.add((dho_n.hasMarshalledPart1, RDFS.subPropertyOf, dho_n.hasMarshalledPart))
    g.add((dho_n.hasMarshalledPart2, RDFS.subPropertyOf, dho_n.hasMarshalledPart))
    g.add((dho_n.hasMarshalledPart3, RDFS.subPropertyOf, dho_n.hasMarshalledPart))
    g.add((dho_n.hasMarshalledPart4, RDFS.subPropertyOf, dho_n.hasMarshalledPart))
    g.add((dho_n.hasMarshalledPart5, RDFS.subPropertyOf, dho_n.hasMarshalledPart))

    g.add((dho_n.hasEscutcheon, RDFS.subPropertyOf, dho_n.numberOfMarshalledParts))

    g.add((dho_n.fromReference, RDF.type, OWL.ObjectProperty))
    g.add((dho_n.hasBlazonTerm, RDF.type, OWL.ObjectProperty))

    g.add((dho_n.hasBeliefValue, RDF.type, OWL.ObjectProperty))
    g.add((dho_n.blazonWasCorrected, RDF.type, OWL.ObjectProperty))
    g.add((dho_n.blazonCorrectionComment, RDFS.subPropertyOf, RDFS.comment))
    g.add((dho_n.correctedCoatOfArms, RDF.type, OWL.ObjectProperty))

def create_initial_classes_and_properties_arm_items(g, dho_n):
    """
    Helper function for the module `map-tblArmItems.py`. Creates classes and properties related to occurences of coats of arms in manuscripts, before
    any data is added to the Knowledge Graph by `map-tblArmItems.py`.
    """
    # Define initial classes
    g.add((dho_n.CoatOfArmsRepresentation, RDF.type, OWL.Class))
    g.add((dho_n.ModeOfCoatOfArmsRepresentation, RDF.type, OWL.Class))
    g.add((dho_n.Shield, RDFS.subClassOf, dho_n.ModeOfCoatOfArmsRepresentation))
    g.add((dho_n.Banner, RDFS.subClassOf, dho_n.ModeOfCoatOfArmsRepresentation))
    g.add((dho_n.TextualBlazon, RDFS.subClassOf, dho_n.ModeOfCoatOfArmsRepresentation))
    g.add((dho_n.Clothing, RDFS.subClassOf, dho_n.ModeOfCoatOfArmsRepresentation))

    g.add((dho_n.CoatOfArmsDescriptionAssignment, RDF.type, OWL.Class))

    # Define initial properties
    g.add((dho_n.hasItemNr, RDF.type, OWL.ObjectProperty))
    g.add((dho_n.locatedOnFolio, RDF.type, OWL.ObjectProperty))
    g.add((dho_n.hasDescriptionAssignment, RDF.type, OWL.ObjectProperty))
    g.add((dho_n.hasPrimaryDescriptionAssignment, RDF.type, dho_n.hasDescriptionAssignment))
    g.add((dho_n.hasBlazon, RDF.type, OWL.ObjectProperty))
    g.add((dho_n.hasPrimaryBlazon, RDFS.subPropertyOf, dho_n.hasBlazon))
    
    g.add((dho_n.hasTimbreAttribute, RDF.type, OWL.ObjectProperty))
    g.add((dho_n.crestPresent, RDFS.subPropertyOf, dho_n.hasTimbreAttribute))
    g.add((dho_n.helmetPresent, RDFS.subPropertyOf, dho_n.hasTimbreAttribute))
    g.add((dho_n.mantlingPresent, RDFS.subPropertyOf, dho_n.hasTimbreAttribute))

    g.add((dho_n.coatOfArmsPresentedAs, RDF.type, OWL.ObjectProperty))
    g.add((dho_n.textHasTranscription, RDF.type, OWL.ObjectProperty))

def create_initial_instances_arm_items(g, dho_n, dho_data_n):
    """
    Helper function for the module `map-tblArmItems.py`. Creates instances, representing necessary metadata of the knowledge graph, before
    any data is added to the Knowledge Graph by `map-tblArmItems.py`.
    """

    # Create people, serving as data annotators
    annotators = [
        'Steen Clemmensen',
        'Torsten Hiltmann',
        'Philipp Schneider'
    ]

    for person in annotators:
        person_uri = URIRef(dho_data_n + person.replace(' ', '-'))
        g.add((person_uri, RDF.type, DCTERMS.Agent))
        g.add((person_uri, RDF.type, OWL.NamedIndividual))