# map-tblArmItems.py
"""
Creates a RDF graph from the OMA table tblArmItems

@author: Philipp Schneider
"""

from rdflib import Graph, URIRef, Literal
from rdflib.namespace import RDF, RDFS, Namespace, OWL, DCTERMS, XSD
import pandas as pd
import uuid
import json

from dho_namespaces import *
import functions
import initial_ontology_definitions

def add_data_to_description_event(description_event_node, coa_node, row, person_uri):
    """
    Adds all information of an occurence of a coat of arms to a description node.
    """

    g.add((description_event_node, dho_n.hasBlazon, coa_node))
    g.add((description_event_node, dho_n.crestPresent, Literal(bool(row['CrestPresent']))))
    g.add((description_event_node, dho_n.helmetPresent, Literal(bool(row['HelmetPresent']))))
    g.add((description_event_node, dho_n.mantlingPresent, Literal(bool(row['MantlingPresent']))))

    # Map mode of visual representation of the coat of arms
    if bool(row['BannerItem']) == True:
        g.add((description_event_node, dho_n.coatOfArmsPresentedAs, dho_n.Banner))
    if bool(row['PersonItem']) == True:
        g.add((description_event_node, dho_n.coatOfArmsPresentedAs, dho_n.Clothing))
    if bool(row['BannerItem']) == False and bool(row['PersonItem']) == False:
        g.add((description_event_node, dho_n.coatOfArmsPresentedAs, dho_n.Shield))

    g.add((description_event_node, DCTERMS.creator, person_uri))

# Load config file
with open("config/config-map-tblArmItems.json", "r") as config_file:
    mapping_config = json.load(config_file)

# Load data from CSV
csv_input_path = mapping_config['csv_input_path']
df_tblArmItems = pd.read_csv(csv_input_path, sep='\t')

# Filter all entries that are already checked but not yet published as RDF
df_tblArmItems = df_tblArmItems[df_tblArmItems['checked_for_publication'] == True]
df_tblArmItems = df_tblArmItems[pd.isnull(df_tblArmItems['published_as_rdf'])]

print(df_tblArmItems)

# Initialize rdflib graph object
g = Graph()

# Bind namespaces
g = bind_namespaces(g)

# Load the existing Knowledge Graph and integrate the new data there, if stated so in the config file
if mapping_config['existing_ontology'] is not None:
    g.parse(mapping_config['existing_ontology'])

# Define initial classes and properties if the respective option is set in the config file
if mapping_config['initial_ontology_definitions'] != None:
    initial_ontology_definitions.create_initial_classes_and_properties_arm_items(g, dho_n)
    initial_ontology_definitions.create_initial_instances_arm_items(g, dho_n, dho_data_n)

# Iterate through CSV to build graph
for index, row in df_tblArmItems.iterrows():
    # Create node for coat of arms reference
    coa_reference_node = URIRef(dho_data_n + 'MS' + str(row['ArmCode']) + '-' + str(row['ArmItemID']))
    g.add((coa_reference_node, RDF.type, dho_n.CoatOfArmsRepresentation))

    g.add((coa_reference_node, dho_n.hasItemNr, Literal(int(row['ItemNr']))))
    g.add((coa_reference_node, dho_n.locatedOnFolio, Literal(str(row['FolNr']))))

    # Create description event
    description_event_node = URIRef(dho_data_n + str(uuid.uuid4()))
    g.add((description_event_node, RDF.type, dho_n.CoatOfArmsDescriptionAssignment))
    g.add((description_event_node, RDF.type, OWL.NamedIndividual))
    g.add((coa_reference_node, dho_n.hasDescriptionAssignment, description_event_node))

    # Link coat of arms reference to coat of arms description
    coa_id = str(row['BranchIDR'])
    coa_node = functions.construct_new_coa_uri(coa_id)

    if row['descriptor_id'] == int(1):
        person_uri = URIRef(dho_data_n + 'Steen-Clemmensen')

        add_data_to_description_event(description_event_node, coa_node, row, person_uri)

        if (coa_reference_node, dho_n.hasPrimaryDescriptionAssignment, None) not in g:
            primary_person_uri = URIRef(dho_data_n + 'Philipp-Schneider')

            # Create new primary node
            primary_description_event_node = URIRef(dho_data_n + str(uuid.uuid4()))
            g.add((primary_description_event_node, RDF.type, dho_n.CoatOfArmsDescriptionAssignment))
            g.add((primary_description_event_node, RDF.type, OWL.NamedIndividual))
            
            g.add((coa_reference_node, dho_n.hasPrimaryDescriptionAssignment, primary_description_event_node))
            g.add((coa_reference_node, dho_n.hasPrimaryBlazon, coa_node))

            add_data_to_description_event(primary_description_event_node, coa_node, row, primary_person_uri)

    elif row['descriptor_id'] == int(2) or pd.isnull(row['descriptor_id']):
        person_uri = URIRef(dho_data_n + 'Philipp-Schneider')

        g.add((coa_reference_node, dho_n.hasPrimaryDescriptionAssignment, description_event_node))
        g.add((coa_reference_node, dho_n.hasPrimaryBlazon, coa_node))

        add_data_to_description_event(description_event_node, coa_node, row, person_uri)
    
    # elif pd.isnull(row['descriptor_id']):
    #     person_uri = URIRef(dho_data_n + 'Philipp-Schneider')

    if not pd.isnull(row['description_time']):
        g.add((description_event_node, DCTERMS.date, Literal(str(row['description_time']), datatype=XSD.date)))

    g.add((coa_reference_node, dho_n.textHasTranscription, Literal(str(row['ArmTxt']))))

    # VORGEHEN
    # ArmItem-ID als Legacy übernehmen
    # daraus (vermutlich) eine eindeutige Nachweis-ID machen (Schema dafür dann, wie mit TH besprochen)
    # auch bei Dopplungen der Nachweis-ID in der Tabelle tblArmItems (Grundlage ist hier ArmItemID) können dann diesem Nachweis-Tripel eines Wappens mehrere Descirption-Events zugewiesen werden
    # WICHTIG: bei BranchIDR noch ein "_1" ans Ende anhängen, ABER nur, wenn von der selben Wappenbeschreibung schon eine event-description erstellt wurde, UND wenn "_1" noch nicht an BranchIDR angehängt wurde

    # hasPrimaryBlason oder so
    # primäre Beschreibung an Anzahl der Beschreibungen entscheiden oder daran, dass Beschreibung schon gesetzt ist


#print(g.serialize())

# Serialize the prepared graph according to options, set in the config file
functions.serialize_graph(g, mapping_config['output_files'])