# map-tblBranch.py
"""
Creates a RDF graph from the OMA tblBranch

@author: Philipp Schneider
"""

from rdflib import Graph, URIRef, Literal
from rdflib.namespace import RDF, RDFS, Namespace, OWL, XSD
import pandas as pd
import uuid
import json

from dho_namespaces import *
import functions
import initial_ontology_definitions
import dho_metadata

def create_owl_class_from_string(new_class_name, sub_class_of=None, create_property=False):
    """
    Takes a string (e.g. from tblBranch table) and creates an OWL class with the
    same name in the dho namespaces. The first letter of the string is capitalized. All 
    white spaces are removed.
    If sub_class_of is set, the new class is also defined as an rdfs:subClassOf of sub_class_of.

    Returns the URI of the newly created class.
    """

    if new_class_name in mapping_config['term_mappings']:
        new_class_name = mapping_config['term_mappings'].get(new_class_name)

    new_class_name = new_class_name.replace("'", '')
    new_class_name = str(new_class_name).title()
    new_class_name = new_class_name.replace(' ', '')

    if create_property == True:
        new_class_name = new_class_name[0].lower() + new_class_name[1:]

    new_class_uri = URIRef(dho_n + new_class_name)

    if sub_class_of is not None:
        if create_property == True:
            g.add((new_class_uri, RDFS.subPropertyOf, sub_class_of))
        else:
            g.add((new_class_uri, RDFS.subClassOf, sub_class_of))
    else:
        g.add((new_class_uri, RDF.type, OWL.Class))

    g.add((new_class_uri, RDFS.label, Literal(new_class_name)))

    return new_class_uri

def create_owl_property_from_string(new_property_name, sub_property_of=None):
    """
    Takes a string (e.g. from tblBranch table) and creates an OWL property with the
    same name in the dho namespaces. All white spaces are removed.
    If sub_property_of is set, the new class is also defined as an rdfs:subPropertyOf of sub_property_of.

    Returns the URI of the newly created property.
    """

    return create_owl_class_from_string(new_property_name, sub_class_of=sub_property_of, create_property=True)

def add_tincture(given_subject, tinct_value, tinct_order_place=None):
    """
    Adds a new tincture node to the node `new_subject`. Takes the value of the
    tincture from a dataframe row, given with `tinct_value`. Also checks if 
    `tinct_value` exists.
    """

    if not pd.isnull(tinct_value):
        new_tinct_node = create_owl_class_from_string(tinct_value)

        # Set predicate for order place
        if tinct_order_place is None:
            predicate = dho_n.hasTincture
        elif tinct_order_place == '1':
            predicate = dho_n.hasTincture1
        elif tinct_order_place == '2':
            predicate = dho_n.hasTincture2
        elif tinct_order_place == '3':
            predicate = dho_n.hasTincture3
        elif tinct_order_place == '4':
            predicate = dho_n.hasTincture4
        
        # Add the new tincture to the graph
        g.add((given_subject, predicate, new_tinct_node))

def add_modifier(charge_node, charge, modifier_number):
    """
    Creates a new node, describing a modifier of a given charge.
    Connects the new modifier to the given charge
    """

    if charge + '_modifier' + modifier_number in df_tblBranch.columns and not pd.isnull(row[charge + '_modifier' + modifier_number]):
        modifier_content = row[charge + '_modifier' + modifier_number]

        modifier_node = URIRef(dho_blazon_n + str(uuid.uuid4()))

        if '(' in modifier_content:
            return None, None, None
            modifier_type = Literal(modifier_content)
        else:
            modifier_type = create_owl_class_from_string(modifier_content, sub_class_of=dho_n.Modifier)
        
        g.add((modifier_node, RDF.type, modifier_type))
        g.add((modifier_node, RDF.type, OWL.NamedIndividual))

        if charge + '_modifier' + modifier_number + '_Tinct' in df_tblBranch.columns:
            add_tincture(modifier_node, row[charge + '_modifier' + modifier_number + '_Tinct'])
        if charge + '_modifier' + modifier_number + '_Tinct1' in df_tblBranch.columns:
            add_tincture(modifier_node, row[charge + '_modifier' + modifier_number + '_Tinct1'], tinct_order_place='1')
        if charge + '_modifier' + modifier_number + '_Tinct2' in df_tblBranch.columns:
            add_tincture(modifier_node, row[charge + '_modifier' + modifier_number + '_Tinct1'], tinct_order_place='2')

        # Link the modifier to the given charge
        g.add((charge_node, dho_n.hasModifier, modifier_node))

def create_visual_link(source_node, target_node=None, direction='current_to_next', charge_number=0):
    """
    Creates a link between two nodes from a column named with the scheme 'Charge#_relation_to_Charge#' (where # is a number).
    The relation is defined as a sub property of dho:hasVisualRelation.
    Target node is newly created and returned, if not given and if the visual link is created. If the visual link is not
    created, `None` is returned.
    """

    valid_directions = {'current_to_next', 'previous_to_current', 'current_to_previous', 'field_relation_to_Layer1'}

    if direction == 'current_to_next':
        relation_column_name = 'Charge' + str(charge_number) + '_relation_to_Charge' + str(charge_number + 1)
    elif direction == 'previous_to_current':
        relation_column_name = 'Charge' + str(charge_number - 1) + '_relation_to_Charge' + str(charge_number)
    elif direction == 'current_to_previous':
        relation_column_name = 'Charge' + str(charge_number) + '_relation_to_Charge' + str(charge_number - 1)
    elif direction == 'from_field':
        relation_column_name = 'field_relation_to_Layer_1'
    else:
        raise ValueError(f'The direction to create a visual link must be of {valid_directions}')

    if target_node is None:
        target_node = URIRef(dho_blazon_n + str(uuid.uuid4()))
            
    if relation_column_name in df_tblBranch.columns and not pd.isnull(row[relation_column_name]):
        relation_property = create_owl_property_from_string(row[relation_column_name], sub_property_of=dho_n.hasVisualRelation)
        g.add((source_node, relation_property, target_node))

        return target_node
    else:
        return None
    
def add_layer(layer_number, layer_content, field_node, previous_layer_node=None, last_charge_from_previous_layer=None, next_layer_node=None):
    """
    Adds a new layer as well as all charges that are stored in this layer, if this layer
    exists in the dataframe.

    Returns the URI of the newly created layer.
    """

    if not pd.isnull(layer_content):
        if next_layer_node is None:
            layer_node = URIRef(dho_blazon_n + str(uuid.uuid4()))
        else:
            layer_node = next_layer_node
        
        next_layer_node = None

        g.add((layer_node, RDF.type, dho_n.Layer))
        g.add((layer_node, RDF.type, OWL.NamedIndividual))
        layer_charge_list = layer_content.split(', ')

        charge_number = 1
        next_charge_node = None

        for charge in layer_charge_list:
            if next_charge_node is None:
                charge_node = URIRef(dho_blazon_n + str(uuid.uuid4()))
            else:
                charge_node = next_charge_node
            
            next_charge_node = None
            if charge != 'MarBlas_FeatureOverall':
                charge_id = int(charge[-1])

            if '{' in row[charge]:
                target_coa = row[charge].split('{')[1].strip()[:-1]
                row[charge] = row[charge].split('{')[0].strip()

                new_sub_coa = functions.construct_new_coa_uri(target_coa)

                if 'escutch' in row[charge]:
                    g.add((new_sub_coa, RDF.type, dho_n.Escutcheon))
                    g.add((new_sub_coa, RDF.type, OWL.NamedIndividual))
                elif 'canton' in row[charge]:
                    g.add((new_sub_coa, RDF.type, dho_n.Canton))
                    g.add((new_sub_coa, RDF.type, OWL.NamedIndividual))
                else:
                    print(row[charge])

                g.add((charge_node, dho_n.hasCoatOfArms, new_sub_coa))

            charge_type = create_owl_class_from_string(row[charge], sub_class_of=dho_n.Charge)
            g.add((charge_node, RDF.type, charge_type))
            g.add((charge_node, RDF.type, OWL.NamedIndividual))
            
            if charge + '_Num' in df_tblBranch.columns and not pd.isnull(row[charge + '_Num']):
                g.add((charge_node, dho_n.hasNumber, Literal(int(row[charge + '_Num']))))

            # Add charge pattern; if no pattern is specified, add Pattern of type dhoh:Plain
            pattern_node = URIRef(dho_blazon_n + str(uuid.uuid4()))

            if charge + '_pattern' in df_tblBranch.columns and not pd.isnull(row[charge + '_pattern']):
                pattern = create_owl_class_from_string(row[charge + '_pattern'], sub_class_of=dho_n.Pattern)
                g.add((pattern_node, dho_n.hasPattern, pattern))

                if charge + '_pattern2' in df_tblBranch.columns and not pd.isnull(row[charge + '_pattern2']):
                    pattern = create_owl_class_from_string(row[charge + '_pattern2'], sub_class_of=dho_n.Pattern)
                    g.add((pattern_node, dho_n.hasPattern, pattern))
            else:
                pattern = dho_n.Plain
                g.add((pattern_node, dho_n.hasPattern, pattern))

            g.add((pattern_node, RDF.type, OWL.NamedIndividual))

            if charge + '_pattern_nr' in df_tblBranch.columns and not pd.isnull(row[charge + '_pattern_nr']):
                g.add((pattern_node, dho_n.hasPatternNumber, Literal(int(row[charge + '_pattern_nr']), datatype=XSD.integer)))
            
            if charge + '_pattern_dir' in df_tblBranch.columns and not pd.isnull(row[charge + '_pattern_dir']):
                pattern_dir = create_owl_class_from_string(row[charge + '_pattern_dir'], sub_class_of=dho_n.Direction)
                g.add((pattern_node, dho_n.hasDirection, pattern_dir))
            
            if charge + '_pattern_line' in df_tblBranch.columns and not pd.isnull(row[charge + '_pattern_line']):
                pattern_line = create_owl_class_from_string(row[charge + '_pattern_line'], sub_class_of=dho_n.Line)
                g.add((pattern_node, dho_n.hasLine, pattern_line))

            # Link pattern to charge
            g.add((charge_node, dho_n.hasPattern, pattern_node))

            # Add tinctures to charge
            add_tincture(pattern_node, row[charge + '_Tinct1'], tinct_order_place='1')

            if charge + '_Tinct2' in df_tblBranch.columns:
                add_tincture(pattern_node, row[charge + '_Tinct2'], tinct_order_place='2')
            if charge + '_Tinct3' in df_tblBranch.columns:
                add_tincture(pattern_node, row[charge + '_Tinct3'], tinct_order_place='3')
            if charge + '_Tinct4' in df_tblBranch.columns:
                add_tincture(pattern_node, row[charge + '_Tinct4'], tinct_order_place='4')

            # Add seme
            if charge + '_pattern_seme' in df_tblBranch.columns and not pd.isnull(row[charge + '_pattern_seme']):
                seme_type = create_owl_class_from_string(row[charge + '_pattern_seme'], sub_class_of=dho_n.Seme)
                g.add((charge_node, dho_n.hasSeme, seme_type))

            if charge + '_pattern_seme2' in df_tblBranch.columns and not pd.isnull(row[charge + '_pattern_seme2']):
                seme_type = create_owl_class_from_string(row[charge + '_pattern_seme2'], sub_class_of=dho_n.Seme)
                g.add((charge_node, dho_n.hasSeme, seme_type))
            
            # Add modifiers and arrangements to charge
            if charge + '_line' in df_tblBranch.columns and not pd.isnull(row[charge + '_line']):
                charge_attribute = create_owl_class_from_string(row[charge + '_line'], sub_class_of=dho_n.Line)
                g.add((charge_node, dho_n.hasLine, charge_attribute))

            if charge + '_dir' in df_tblBranch.columns and not pd.isnull(row[charge + '_dir']):
                charge_attribute = create_owl_class_from_string(row[charge + '_dir'], sub_class_of=dho_n.Direction)
                g.add((charge_node, dho_n.hasDirection, charge_attribute))

            if charge + '_arrangement' in df_tblBranch.columns and not pd.isnull(row[charge + '_arrangement']):
                arrangement_node = URIRef(dho_blazon_n + str(uuid.uuid4()))
                charge_attribute = create_owl_class_from_string(row[charge + '_arrangement'], sub_class_of=dho_n.Arrangement)
                g.add((arrangement_node, RDF.type, charge_attribute))
                g.add((arrangement_node, RDF.type, OWL.NamedIndividual))

                if row[charge + '_arrangement'] == 'orle of':
                    g.add((charge_node, dho_n.hasNumber, dho_n.Multiple))

                if charge + '_arrangement_dir' in df_tblBranch.columns and not pd.isnull(row[charge + '_arrangement_dir']):
                    charge_attribute = create_owl_class_from_string(row[charge + '_arrangement_dir'], sub_class_of=dho_n.Direction)
                    g.add((arrangement_node, dho_n.hasDirection, charge_attribute))

                g.add((charge_node, dho_n.arranged, arrangement_node))
            
            if charge + '_hasText' in df_tblBranch.columns and not pd.isnull(row[charge + '_hasText']):
                g.add((charge_node, dho_n.hasText, Literal(str(row[charge + '_hasText']), datatype=XSD.string)))
            
            if charge + '_hasLetter' in df_tblBranch.columns and not pd.isnull(row[charge + '_hasLetter']):
                g.add((charge_node, dho_n.hasLetter, Literal(str(row[charge + '_hasLetter']), datatype=XSD.string)))
            
            add_modifier(charge_node, charge, '1')
            add_modifier(charge_node, charge, '2')
            add_modifier(charge_node, charge, '3')
            add_modifier(charge_node, charge, '4')

            # Create visual link to field
            if charge + '_relation_to_field' in df_tblBranch.columns and not pd.isnull(row[charge + '_relation_to_field']):
                visual_relation = create_owl_property_from_string(row[charge + '_relation_to_field'], sub_property_of=dho_n.hasVisualRelation)
                g.add((charge_node, visual_relation, field_node))

            # Create visual link from current Charge to next Charge if next Charge is part of the same Layer
            if charge != layer_charge_list[-1]:
                next_charge_node = create_visual_link(charge_node, direction='current_to_next', charge_number=charge_id)

            # Create visual link from current Charge to previous Charge if previous Charge is part of the same Layer
            if charge != layer_charge_list[0]:
                next_charge_node = create_visual_link(charge_node, previous_charge_node, direction='current_to_previous', charge_number=charge_id)

            # Link new charge to layer
            charge_property = URIRef(dho_n + 'hasCharge' + str(charge_number))
            g.add((charge_property, RDFS.subPropertyOf, dho_n.hasCharge))
            g.add((layer_node, charge_property, charge_node))

            charge_number += 1
            previous_charge_node = charge_node
        
        # Create visual link from current Layer to next Layer
        if layer_number != '4' and charge != 'MarBlas_FeatureOverall':
            next_layer_node = create_visual_link(layer_node, direction='current_to_next', charge_number=charge_id)

        # Create visual link from current Layer to previous Layer
        if layer_number != '1' and charge != 'MarBlas_FeatureOverall':
            next_layer_node = create_visual_link(layer_node, previous_layer_node, direction='current_to_previous', charge_number=charge_id)

        # Link Layer to new coa description
        layer_property = URIRef(dho_n + 'hasLayer' + str(layer_number))
        g.add((new_coa_description, layer_property, layer_node))

        return layer_node, charge_node, next_layer_node

    else:
        return None, None, None

# Load config file
with open("config/config-map-tblBranch.json", "r") as config_file:
    mapping_config = json.load(config_file)

# Load data from CSV
csv_input_path = mapping_config['csv_input_path']
df_tblBranch = pd.read_csv(csv_input_path, sep='\t')

# Filter all entries that are already checked but not yet published as RDF
df_tblBranch = df_tblBranch[(df_tblBranch['checked_for_publication'] == "true") | (df_tblBranch['checked_for_publication'] == 'secondary-description')]
df_tblBranch = df_tblBranch[pd.isnull(df_tblBranch['published_as_rdf'])]

print(df_tblBranch)

# Initialize rdflib graph object
g = Graph()

# Load the existing Knowledge Graph and integrate the new data there, if stated so in the config file
if mapping_config['existing_ontology'] is not None:
    g.parse(mapping_config['existing_ontology'])
else:
    # Bind namespaces
    g = bind_namespaces(g)

# Define initial classes and properties if the respective option is set in the config file
if mapping_config['initial_ontology_definitions'] != None:
    g.add((dho_uri, RDF.type, OWL.Ontology))
    initial_ontology_definitions.create_initial_classes_and_properties_heraldry(g, dho_n)

# Add metadata, defined in `dho_metadata.py`, if set in the config file
if mapping_config['add_metadata'] == True:
    dho_metadata.add_ontology_metadata(g)

# Iterate through CSV to build graph
for index, row in df_tblBranch.iterrows():
    # New coa description
    branch_id = str(row['BranchID'])

    if '.0' in branch_id:
        branch_id = branch_id.split('.', 1)[0]

    new_coa_description = functions.construct_new_coa_uri(branch_id)

    if (new_coa_description, RDF.type, None) not in g:
        g.add((new_coa_description, RDF.type, dho_n.CoatOfArms))
        g.add((new_coa_description, RDF.type, OWL.NamedIndividual))

    g.add((new_coa_description, dho_n.hasOMAid, Literal(str(branch_id), datatype=XSD.string)))

    # Add Belief Values
    # if row['checked_for_publication'] == 'true':
    #     g.add((new_coa_description, dho_n.hasBeliefValue, Literal(True, datatype=XSD.boolean)))
    # elif row['checked_for_publication'] == 'secondary-description':
    #     g.add((new_coa_description, dho_n.hasBeliefValue, Literal(False, datatype=XSD.boolean)))
    
    if not pd.isnull(row['blason_corrected']):
    #     g.add((new_coa_description, dho_n.blazonWasCorrected, Literal(True, datatype=XSD.boolean)))

        # if not pd.isnull(row['comment_blason_correction']):
        #     g.add((new_coa_description, dho_n.blazonCorrectionComment, Literal(str(row['comment_blason_correction']), datatype=XSD.string)))
        
        # Link to legacy coat of arms description
        legacy_coa = functions.construct_new_coa_uri(branch_id + '_1')
        g.add((legacy_coa, RDF.type, dho_n.CoatOfArms))
        g.add((legacy_coa, RDF.type, OWL.NamedIndividual))
        g.add((new_coa_description, dho_n.correctedCoatOfArms, legacy_coa))

    # Create field
    field_node = URIRef(dho_blazon_n + str(uuid.uuid4()))
    g.add((field_node, RDF.type, dho_n.Field))
    g.add((field_node, RDF.type, OWL.NamedIndividual))
    # Link field to new coa description
    g.add((new_coa_description, dho_n.hasField, field_node))

    if not pd.isnull(row['field']):
        pattern = create_owl_class_from_string(row['field'], sub_class_of=dho_n.Pattern)
        g.add((field_node, dho_n.hasPattern, pattern))
    
    if not pd.isnull(row['field_style_modifier']):
        pattern = create_owl_class_from_string(row['field_style_modifier'], sub_class_of=dho_n.Pattern)
        g.add((field_node, dho_n.hasPattern, pattern))

    if not pd.isnull(row['field_dir']):
        field_dir = create_owl_class_from_string(row['field_dir'], sub_class_of=dho_n.Direction)
        g.add((field_node, dho_n.hasDirection, field_dir))

    if not pd.isnull(row['field_line']):
        field_line = create_owl_class_from_string(row['field_line'], sub_class_of=dho_n.Line)
        g.add((field_node, dho_n.hasLine, field_line))
    
    if not pd.isnull(row['field_line_modifier']):
        field_line_mod = create_owl_class_from_string(row['field_line_modifier'], sub_class_of=dho_n.LineModifier)
        g.add((field_node, dho_n.hasLineModifier, field_line_mod))

    for style_modifier_id in range(1, 3):
        if not pd.isnull(row['field_style' + str(style_modifier_id)]):
            field_style_mod = create_owl_class_from_string(row['field_style' + str(style_modifier_id)], sub_class_of=dho_n.Modifier)
            g.add((field_node, dho_n.hasModifier, field_style_mod))

    if not pd.isnull(row['field_nr']):
        g.add((field_node, dho_n.hasNumber, Literal(int(row['field_nr']), datatype=XSD.integer)))

    # Add field tinctures
    add_tincture(field_node, row['field_TinctPlain'])
    add_tincture(field_node, row['field_Tinct1'], tinct_order_place='1')
    add_tincture(field_node, row['field_Tinct2'], tinct_order_place='2')
    add_tincture(field_node, row['field_Tinct3'], tinct_order_place='3')

    # Create seme for field
    if not pd.isnull(row['field_seme']):
        field_seme_node = URIRef(dho_blazon_n + str(uuid.uuid4()))
        seme_pattern = create_owl_class_from_string(row['field_seme'], sub_class_of=dho_n.Seme)

        g.add((field_seme_node, RDF.type, dho_n.Seme))
        g.add((field_seme_node, RDF.type, OWL.NamedIndividual))
        g.add((field_seme_node, dho_n.hasSeme, seme_pattern))
        add_tincture(field_seme_node, row['field_seme_Tinct'])
        # seme_tincture = create_owl_class_from_string(row['field_seme_Tinct'], sub_class_of=dho_n.Tincture)
        # g.add((field_seme_node, dho_n.hasTincture, seme_tincture))

        # Link seme to field
        g.add((field_node, dho_n.hasSeme, field_seme_node))
    
    # Link to marshalled Arms
    if not pd.isnull(row['Qtrs']) and row['Qtrs'] != 1:
        g.add((field_node, dho_n.numberOfMarshalledParts, Literal(int(float(row['Qtrs'])), datatype=XSD.integer)))
    
    if not pd.isnull(row['MarBlas_Type']):
        mar_type_class = create_owl_class_from_string(row['MarBlas_Type'], sub_class_of=dho_n.MarshallingType)
        g.add((field_node, dho_n.marshalledByType, mar_type_class))
    elif pd.isnull(row['MarBlas_Type']) and int(row['Qtrs']) == 4:
        g.add((field_node, dho_n.marshalledByType, dho_n.PerQuarter))
    
    for mar_number in range(1, 6):
        if not pd.isnull(row['MarBlas_' + str(mar_number) + 'ID']):
            new_mar_coa = functions.construct_new_coa_uri(row['MarBlas_' + str(mar_number) + 'ID'])
            mar_connect_property = URIRef(dho_n + 'hasMarshalledPart' + str(mar_number))
            g.add((field_node, mar_connect_property, new_mar_coa))
            g.add((new_mar_coa, RDF.type, dho_n.CoatOfArms))
            g.add((new_mar_coa, RDF.type, OWL.NamedIndividual))

    # Create Layers
    layer_1, last_charge, next_layer = add_layer('1', row['Layer1'], field_node)
    create_visual_link(field_node, layer_1, 'from_field')
    create_visual_link(field_node, last_charge, 'from_field')
    layer_2, last_charge, next_layer = add_layer('2', row['Layer2'], field_node, previous_layer_node=layer_1, last_charge_from_previous_layer=last_charge, next_layer_node=next_layer)
    layer_3, last_charge, next_layer = add_layer('3', row['Layer3'], field_node, previous_layer_node=layer_2, last_charge_from_previous_layer=last_charge, next_layer_node=next_layer)
    layer_4, last_charge, next_layer = add_layer('4', row['Layer4'], field_node, previous_layer_node=layer_3, last_charge_from_previous_layer=last_charge)

    # Link to Escutcheon, layed over the whole coat of arms
    if not pd.isnull(row['MarBlas_EscuID']):
        new_escu_coa = functions.construct_new_coa_uri(row['MarBlas_EscuID'])
        escu_type = create_owl_class_from_string(row['Escutcheon_Type'], sub_class_of=dho_n.Escutcheon)
        g.add((new_escu_coa, RDF.type, escu_type))
        g.add((new_escu_coa, RDF.type, OWL.NamedIndividual))
        g.add((new_coa_description, dho_n.hasEscutcheon, new_escu_coa))

    # Create Charge, layed over all (marshalled) parts of the coat of arms
    if not pd.isnull(row['MarBlas_FeatureOverall']):
        layer_overall, last_charge, next_layer = add_layer('Overall', 'MarBlas_FeatureOverall', field_node)

#print(g.serialize())

# Serialize the prepared graph according to options, set in the config file
functions.serialize_graph(g, mapping_config['output_files'])


#g = Graph()
#g.parse('/Users/pschneider/Documents/Digital Heraldry/Repositorien/dho-knowledge-graph-data-integration/digital-heraldry-ontology/ontology.ttl')
# g.remove((None, RDF.type, OWL.NamedIndividual))
# g.remove((None, RDF.type, dho_n.CoatOfArms))

##

# Create additional ontology file as TBox by deleting all entities that are part of the namespace(s) for named individuals
print('Export TBox to file...')
for subject, predicate, object in g.triples((None, None, None)):
    assert isinstance(predicate, URIRef)
    #if subject.startswith(dho_blazon_n):
    if 'digital-heraldry-ontology/blazon' in subject:
        g.remove((subject, predicate, object))

print('Create data/rdf-output/digital-heraldry-ontology.ttl')
g.serialize(destination='data/rdf-output/digital-heraldry-ontology.ttl', format='ttl')