# update_documentation.py
"""
Integrates documentation of classes and properties from tsv-tables into a given RDF file.
"""

from rdflib import Graph, URIRef, Literal
from rdflib.namespace import RDF, RDFS, Namespace, XSD, OWL, SKOS
import pandas as pd
import argparse
from dho_namespaces import *

upper_level_docu_path = 'data/input/documentation/dho-upperlevel-documentation.tsv'
field_seme_docu_path = 'data/input/documentation/dho-field-and-seme.tsv'

reference_list = [
    {
        "reference": "de Boos",
        "reference_title": "de Boos, Emmanuel: Dictionnaire du Blason, Paris 2001.",
    },
    {
        "reference": "Rietstap",
        "reference_title": "Rietstap, Jean Baptiste: Armorial Général. Contenant la Description des Armoirries des Familles nobles et patriciennes de l'Europe; Précédé d'un Dictionnaire des Termes du Blason, Gouda 1861."
    },
    {
        "reference": "Oswald",
        "reference_title": "Oswald, Gert: Lexikon der Heraldik. Von Apfelkreuz bis Zwillingsbalken, 3., unveränd. Aufl., Regenstauf 2011.",
    },
    {
        "reference": "Gritzner",
        "reference_title": "Gritzner, Maximilian: Handbuch der heraldischen Terminologie in zwölf (germanischen und romanischen) Zungen. Enthaltend zugleich die Haupt-Grundsätze zur Wappenkunst (J. Siebmachers grosses und allgemeines Wappenbuch, Einleitungsband, Abt. B); Nürnberg 1890.",
    },
    {
        "reference": "Parker",
        "reference_title": "Parker, James: A Glossary of Terms used in Heraldry, Oxford, London 1894.",
    },
    {
        "reference": "Fox-Davies",
        "reference_title": "Fox-Davies, Arthur Charles: A Complete Guide to Heraldry, London, Edinburgh 1909.",
    }
]

def add_definition_from_bib_reference(g, entity_node, entity_name, reference, reference_title):
    """
    Adds a definition and labels for an entry on the bibliographic record `reference_title` as a documenting node
    to `entity_node` in graph `g`. `reference` must be the abbreviation for the bibliographic record, used in 
    the documentation table.
    """

    if not pd.isnull(row[reference_title]) and row[reference_title] != 'not defined':
        reference_node_name = reference.replace(' ', '')
        reference_node = URIRef(dho_n + entity_name + reference_node_name)
        g.add((reference_node, RDFS.label, Literal(entity_name + reference_node_name)))

        g.add((entity_node, RDFS.isDefinedBy, reference_node))

        g.add((reference_node, RDF.type, dho_n.BlazonTermDefinition))
        g.add((reference_node, RDF.type, OWL.NamedIndividual))

        definition_text = row[reference_title]
        g.add((reference_node, RDFS.comment, Literal(definition_text)))
        g.add((reference_node, dho_n.fromReference, Literal(reference_title)))

        # Add multilingual labels
        label_list = row[reference + '_labels'].split(', ')

        for label in label_list:
            label_content = label.split(': ')
            label_name = label_content[1]
            label_language = label_content[0]

            g.add((reference_node, dho_n.hasBlazonTerm, Literal(label_name, lang=label_language)))

if __name__ == '__main__':
    parser = argparse.ArgumentParser(
        description=__doc__,
        formatter_class=argparse.RawDescriptionHelpFormatter
    )

    parser.add_argument(
        'input_rdf_file',
        type=str,
        help='One or rdf file into which the documentation from all tsv-files in data/input/documentation is to be included'
    )

    args = parser.parse_args()
    rdf_file_path = args.input_rdf_file

    # Initialize rdflib graph object
    g = Graph()
    g.parse(rdf_file_path)

    # Bind namespaces
    g = bind_namespaces(g)

    # Load documentation for upper level classes and properties
    df_upperlevel_docu = pd.read_csv(upper_level_docu_path, sep='\t')

    print('Integrate documentation for upper level classes and properties')
    for index, row in df_upperlevel_docu.iterrows():
        current_entity = URIRef(dho_n + row['entity_name'])
        g.add((current_entity, RDFS.label, Literal(row['entity_name'], lang='en')))

        # Overwrite all previous documentations
        if (current_entity, RDFS.comment, None) in g:
            g.remove((current_entity, RDFS.comment, None))
            
        g.add((current_entity, RDFS.comment, Literal(row['comment_en'], lang='en')))

    # Load documentation for field and seme
    print(f'Integrate documentation for classes and properties descriped in {field_seme_docu_path}')
    df_field_seme_docu = pd.read_csv(field_seme_docu_path, sep='\t')

    for index, row in df_field_seme_docu.iterrows():
        entity_name = ''.join(x for x in row['entity_name'].title() if not x.isspace())
        entity_node = URIRef(dho_n + entity_name)
        
        if (entity_node, None, None) in g:
            g.add((entity_node, RDFS.comment, Literal('comment', lang='en')))

            for definition_entry in reference_list:
                add_definition_from_bib_reference(g, entity_node, entity_name, definition_entry['reference'], definition_entry['reference_title'])


    # Export ontology to file
    g.serialize(destination=rdf_file_path, format='ttl')